#
# Copyright (c) 2008-2018 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

from persistent import Persistent
from zope.container.contained import Contained
from zope.interface import Interface, implementer
from zope.location import ILocation
from zope.schema.fieldproperty import FieldProperty

from pyams_content.features.search import ISearchFolder
from pyams_content.features.search.portlet import ISearchResultsPortletSettings
from pyams_content.shared.common import IWfSharedContent
from pyams_content.shared.common.portlet.interfaces import ISharedContentHeadViewletManager
from pyams_default_theme.features.search.portlet.interfaces import ISearchResultRenderer, \
    ISearchResultsPortletDefaultRendererSettings
from pyams_default_theme.interfaces import ISearchResultsView
from pyams_portal.interfaces import IPortalContext, IPortletRenderer
from pyams_portal.portlet import PortletRenderer
from pyams_skin.interfaces.viewlet import IBreadcrumbs
from pyams_skin.layer import IPyAMSLayer, IPyAMSUserLayer
from pyams_template.template import template_config
from pyams_utils.adapter import NullAdapter, adapter_config
from pyams_utils.factory import factory_config
from pyams_utils.url import canonical_url, relative_url
from pyams_viewlet.viewlet import ViewContentProvider, Viewlet, viewlet_config

from pyams_default_theme import _


#
# Search folder custom head specificities renderer
#

@viewlet_config(name='search-folder-head', context=ISearchFolder, layer=IPyAMSUserLayer, view=Interface,
                manager=ISharedContentHeadViewletManager, weight=1)
@template_config(template='templates/folder-head-specificities.pt', layer=IPyAMSUserLayer)
class SearchFolderHeadViewlet(Viewlet):
    """Search folder head specificities viewlet"""


#
# Search results portlet renderers
#

@factory_config(provided=ISearchResultsPortletDefaultRendererSettings)
class SearchResultsPortletDefaultRendererSettings(Persistent, Contained):
    """Search results portlet default renderer settings"""

    display_results_count = FieldProperty(ISearchResultsPortletDefaultRendererSettings['display_results_count'])
    allow_sorting = FieldProperty(ISearchResultsPortletDefaultRendererSettings['allow_sorting'])
    allow_pagination = FieldProperty(ISearchResultsPortletDefaultRendererSettings['allow_pagination'])


@adapter_config(context=(IPortalContext, IPyAMSLayer, Interface, ISearchResultsPortletSettings),
                provides=IPortletRenderer)
@template_config(template='templates/search-results.pt', layer=IPyAMSLayer)
@implementer(ISearchResultsView)
class SearchResultsPortletDefaultRenderer(PortletRenderer):
    """Search results portlet default renderer"""

    label = _("Default search results")

    settings_interface = ISearchResultsPortletDefaultRendererSettings

    def update(self):
        settings = self.renderer_settings
        if not settings.allow_pagination:
            self.request.GET['length'] = '999'
        super(SearchResultsPortletDefaultRenderer, self).update()

    def render_item(self, item):
        renderer = self.request.registry.queryMultiAdapter((item, self.request, self), ISearchResultRenderer)
        if renderer is not None:
            renderer.update()
            return renderer.render()
        else:
            return ''


@adapter_config(context=(ILocation, IPyAMSUserLayer, ISearchResultsView), provides=IBreadcrumbs)
class BreadcrumbsAdapter(NullAdapter):
    """Disable breadcrumbs in search results view"""


@adapter_config(context=(IWfSharedContent, IPyAMSUserLayer, ISearchResultsView), provides=ISearchResultRenderer)
@template_config(template='templates/search-result.pt', layer=IPyAMSUserLayer)
class WfSharedContentSearchResultRenderer(ViewContentProvider):
    """Shared content search result renderer"""

    def get_url(self, target):
        if self.view.settings.force_canonical_url:
            return canonical_url(target, self.request)
        else:
            return relative_url(target, self.request)
